/******************************************************************/
/* james - LCD.C library - T6963C display driver 64 x 128 pixels  */
/******************************************************************/
#include <c_ext.h>
#include <absacc.h>
#include <string.h>
#include <intrins.h>

#include <lcd.h>     // usefull type codes font, display modes, address etc. 

 uchar code mask[] = { 0, 0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE, 0xFF };

// LCDT variables 
 uchar idata x0;        // horizzontal position 0..Xmax-1 
 uchar idata y0;        // vertical position 0..Ymax-1
 uchar idata status;    // page - mode - font

// LCD 4 pages buffer + 1 row
 uchar xdata bmx [3 * Ymax * Nmax + Nmax];

// font pointers
 uchar code * xdata p5x8 [256];

// bit status mask
#define  TMASK   0x80             // display model mask
#define  FMASK   0x70             // font mask
#define  MMASK   0x08             // mode mask
#define  PMASK   0x07             // page mask

#define  display_model  (status & TMASK)
#define  display_mode   (status & MMASK)
#define  selected_font  (status & FMASK)
#define  selected_page  (status & PMASK)

#define  DY  ((display_model == OEMDISP) ? 1 : 0)


uint address_pointer_corrente ()
/**************************************************************/
{
 return 1024 * selected_page + y0*Nmax + x0/8;
}


void send_cmd (uchar cmd)
/**************************************************************/
{
 while(ready_to_write != (ready_to_write & XBYTE[LCD_cmd]));
 XBYTE[LCD_cmd] = cmd;
}


void send_dato (uchar dato)
/**************************************************************/
{
 while(ready_to_write != (XBYTE[LCD_cmd] & ready_to_write));
 XBYTE[LCD_dato] = dato;
}


void auto_write (uchar dato)
/**************************************************************/
{
 while((XBYTE[LCD_cmd] & ready_to_autowrite) == 0);
 XBYTE[LCD_dato] = dato;
}


void send_cmd2 (uchar cmd, uint dato)
/**************************************************************/
{
 send_dato(byte_basso(dato));
 send_dato(byte_alto(dato));
 send_cmd(cmd);
}


void gotoxy (uchar x, uchar y)
/**************************************************************/
{
 x0 = x;
 y0 = y;
}


void set_display (uchar font, uchar mode)
/**************************************************************/
{
 status = (status & ~(MMASK|FMASK)) | (mode & MMASK) | (font & FMASK); 
}


void wrbm (uchar w, uchar h, uchar * bm)
/**************************************************************/
/* writes character in the display zone limited by 			  */	
/* box (x0, y0, x0+w, y0+h)	and bm sets the font              */
/**************************************************************/
{
 static uchar i, j, k, a, b, c, d, n, m0, m1, ncbm;
 uint aa, bb;
 ncbm = 1 + (w-1) / 8;        	// bm[] build ncbm * h bytes
 k = x0 + w - 1;
 m0 = mask[x0 % 8];             // first coloumn mask
 m1 = mask[1 + (k % 8)];        // last coloumn mask
 n = 1 + k/8 - x0/8;            // max num of coloumns
 aa = address_pointer_corrente();
 
// clipping if h and n are greather than Ymax and Nmax
 if (h > Ymax - y0) h = Ymax - y0;
 if (n > Nmax - x0/8) n = Nmax - x0/8; 

// 1 pixel rows loop
 i = 0; do
   {
   // set pixel without modify x0 and y0 
    bb = aa + i*Nmax;
   // 8 pixels coloumns loop 
    k = 0; while (k < n)
      {
       if (k) b = c;
       j = ncbm*i + k;
       c = _cror_(bm[j], x0 % 8);
       a = c; 
       if (display_mode == REVERSE) { a = ~a; b = ~b; }
       a &= ~m0;
      // current contents of the first and last coloumns have  
      // to be restored -> read and save 
       d = bmx[bb];
      // writes, recovering if required what has been saved
       bmx[bb++] =
         // just one coloumn... 
	   (n == 1) ? (a & m1) | (d & (m0|~m1)) :
         // first coloumn of 2 or more...
	   (k == 0) ? a | (d & m0) :
         // last coloumn of 2 more...
	   (k == n-1) ?
              (ncbm == k) ? (b & m1) | (d & (~m1)) :
              (b & m0) | (a & m1) | (d & (~m1)) :
         // intermediat coloumn of 3 or more...
	    a | (b & m0);
       k++;
      }
   }
 while (++i < h);

// set pixel for next writing if required
 x0 = (x0 + w < Xmax) ? x0 + w : Xmax; 
}


void wrch (uchar ch)
/*************************************************************/
{
 switch (selected_font)
   {
    case F4x5:
       wrbm((ch < 10) ? 8 : 5, 5, bm4x5(ch)); break;
    case F5x6:
       wrbm((ch < 10) ? 13 : 6, 6, p5x8[ch] + 1); break;
    case F5x8:
       wrbm((ch < 10) ? 13 : 6, 8, p5x8[ch]); break;
    case F10x12:
       wrbm((ch == '.') ? 5 : (ch < 10) ? 24 : 12, 12, bm10x12(ch)); break;
    case F15x18:
       wrbm((ch == '.') ? 7 : 18, 18, bm15x18(ch)); break;
   }
}


uchar ch_width ()
/*************************************************************/
{
 switch (selected_font)
   {
    case F4x5:   return 5;
    case F5x6:
    case F5x8:   return 6;
    case F10x12: return 12;
    case F15x18: return 18;
   }
}


uchar dp_width ()
/*************************************************************/
{
 switch (selected_font)
   {
    case F4x5:   return 5;
    case F5x6:
    case F5x8:   return 6;
    case F10x12: return 5;
    case F15x18: return 7;
   }
}


uchar ch_height ()
/*************************************************************/
{
 switch (selected_font)
   {
    case F4x5: return 5;
    case F5x6: return 6;
    case F5x8: return 8;
    case F10x12: return 12;
    case F15x18: return 18;
   }
}


void wrstr (char* s)
/*************************************************************/
{
 while (*s) wrch(*s++);
}


void clearfield (uchar w, uchar h)
/**************************************************************/
/* erase all the contents in the box (x0, y0, x0+w, y0+h)     */
/* and when finished set graphical cursor to home (x0, y0)    */
/**************************************************************/
{
 code uchar blank[] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
 uchar xs = x0;
 uchar ys = y0;
 while (h--) { x0 = xs; y0 = ys + h; wrbm(w, 1, blank); }
 x0 = xs; y0 = ys;
}


void clearto (uchar x, uchar y)
/**************************************************************/
/* erase all the contents in the box (x0, y0, x, y)           */
/* and then set cursor to right							      */
/**************************************************************/
{
 clearfield(x - x0, y - y0);
 x0 = x;
}


void set_pixel (uchar x, uchar y)
/**************************************************************/
{
 x0 = x; y0 = y;
 bmx[address_pointer_corrente()] |= 1 << (7&(~x));
}



void hline (char dx)
/**************************************************************/
/* draw an horizzontal line of |dx|+1 pixels starting from    */
/* the current home position (x0,y0).                         */
/* New curren position = end of line				          */
/**************************************************************/ 
{
 status ^= MMASK;
 if (dx >= 0) 
   {
    clearfield(1+(uchar)dx, 1);
    gotoxy(x0+(uchar)dx, y0);
   }
 else 
   {
    dx = -dx;
    gotoxy(x0-(uchar)dx, y0);
    clearfield(1+(uchar)dx, 1);
   }
 status ^= MMASK;
} 


void vline (char dy)
/**************************************************************/
/* draw a vertical line of |dy|+1 pixels starting from        */
/* the current home position (x0,y0).                         */
/* New curren position = end of line				          */
/**************************************************************/ 
{
 uchar m = 1 << (7&(~x0));
 uint aa = address_pointer_corrente();
 y0 += dy;
 if (dy >= 0) do 
   {
    if (status & MMASK) bmx[aa] &= ~m; else bmx[aa] |= m;
    aa += Nmax;
   }
 while (dy--);
 else do
   {
    if (status & MMASK) bmx[aa] &= ~m; else bmx[aa] |= m;
    aa -= Nmax;
   }
 while (dy++);        
} 



void box (uchar dx, uchar dy)
/**************************************************************/
/* draw a box with dimension of dx+1 and  dy+1 pixels         */
/* valid values: dx = 0..127, dy = 0..63                      */
/* (dx = dy = 0 is an unefficient way to draw a pixel         */
/**************************************************************/
{
 hline(dx);
 vline(dy);
 hline(-(char)dx);
 vline(-(char)dy);
}



void clearlcd (uchar zona)
/**************************************************************/
{ 
 uint aa;
 uchar xin, yin, nx, ny, i;
 xin = (zona & LEFT) ? 0 : 64;
 yin = (zona & UPPER) ? 0 : 32;
 nx = ((zona & RIGHT) && (zona & LEFT)) ? 16 : 8;
 ny = ((zona & UPPER) && (zona & LOWER)) ? 64 : 32;
 while (ny--)
   {
    x0 = xin;
    y0 = yin + ny;
    aa = address_pointer_corrente();
    i = nx; while (i--) bmx[aa+i] = 0;
   }
 x0 = xin; y0 = yin;
}


void set_page (uchar page)
/**************************************************************/
{
 status = (status & ~PMASK) | (page & PMASK);
}


void goto_page (uchar page)
/**************************************************************/
{
 send_cmd2(graphic_home_address, GHOME + 1024*(page & PMASK));
}



void copy_page (uchar src_page, uchar dest_page)
/**************************************************************/
{
 memcpy(&bmx[1024*(uint)dest_page], &bmx[1024*(uint)src_page], 1024); 
}


void refresh_LCD (uchar page)
/**************************************************************/
{
 uint i = 1024 * (uint)page;
 uint imax = i + 1024;
 send_cmd2(address_pointer, GHOME + i + Nmax*DY);
 send_cmd(autowrite_ON);
 do { auto_write(bmx[i]); } while (++i < imax);
 send_cmd(auto_OFF);
 goto_page(page);
}


void save_current_page (uchar dest_page)
/**************************************************************/
{
 copy_page(status & PMASK, dest_page);
}


void init_LCD (uchar tipo)
/**************************************************************/
{
 uchar ch;
 status = tipo;
 send_cmd2 (graphic_area, 16); 			// 16 coloumns of 8 pixels
 send_cmd  (0x80);             			// OR mode, internal char gen.
 send_cmd  (graphic_ON);
 set_display (F5x8, NORMAL);
 set_page(0);
 clearlcd(ALL);
 refresh_LCD(0);
 goto_page(0);

 ch = 255; do 
   {
    p5x8[ch] = bm5x8(ch);
   }
 while (ch--);
}
